@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+﻿# -*- coding: utf-8 -*-
 # Sleeper Desktop Widget for Maemo 5
 # Copyright (c) 2010 The Authors
 #
@@ -10,7 +10,7 @@
 # python2.5 /usr/lib/hildon-desktop/sleeper.py
 
 APP_TITLE = "Sleeper"
-APP_VERSION = "1.1"
+APP_VERSION = "1.2"
 
 import gtk
 import pygtk
@@ -23,6 +23,7 @@ import gobject
 import gconf
 import osso
 import string
+import subprocess
 
 APP_NAME = APP_TITLE.lower()
 LOGO = '/usr/share/icons/hicolor/48x48/hildon/%s.png' % APP_NAME
@@ -38,12 +39,18 @@ class Panucci(object):
     DBUS_PATH = '/panucciInterface'
     DBUS_INTF = 'org.panucci.panucciInterface'
 
+class FMRadio(object):
+    DBUS_NAME = 'de.pycage.fmradio'
+    DBUS_PATH = '/de/pycage/fmradio'
+    DBUS_INTF = 'de.pycage.fmradio'
 
+    
 class SleeperHomePlugin(hildondesktop.HomePluginItem):
     defaultPRESETS = [15,30,60]
     defaultLAYOUT = 'H'   #Or 'V' for a vertical bar
     defaultICON = True
     defaultSHOWBANNER = True
+    defaultSHUTDOWN = False
      
     BUTTON_SIZE = 64
     FIRSTRUN = True
@@ -57,6 +64,7 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
         self.connect("show-settings", self.show_options)
     
         self.bus = dbus.SessionBus()
+        self.systembus = dbus.SystemBus()
         self.timer = None
         
         #Load the settings using GConf
@@ -66,6 +74,7 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
         self.LAYOUT=conf_client.get_string(GCONFPATH+"/layout")
         self.ICON=conf_client.get_bool(GCONFPATH+"/drawicon")
         self.SHOWBANNER=conf_client.get_bool(GCONFPATH+"/showbanner")
+        self.SHUTDOWN=conf_client.get_bool(GCONFPATH+"/shutdown")
 
         #Fallback to default on first launch
         if not self.PRESETS:
@@ -76,6 +85,8 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
           self.ICON=self.defaultICON    
         if self.SHOWBANNER == None:
           self.SHOWBANNER=self.defaultSHOWBANNER  
+        if self.SHUTDOWN == None:
+          self.SHUTDOWN=self.defaultSHUTDOWN  
         #Draw the widget
         self.draw()
 
@@ -135,7 +146,7 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
             for child in self.box.get_children():
                 if isinstance(child, gtk.ToggleButton) and child != button:
                     child.set_active(False)
-
+            # Multiply with 60 to get minutes, multiply with 1 to keep seconds (useful for testing)
             self.timer = gobject.timeout_add_seconds(minutes*60, \
                     self.on_sleeptimer_fired, minutes, button)
 
@@ -148,11 +159,43 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
         button.set_active(False)
         self.timer = None
 
+        # Get current volume
+        obj = self.bus.get_object("com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer", "/com/nokia/mafw/renderer/gstrenderer")
+        mafw = dbus.Interface(obj, "com.nokia.mafw.extension")
+        volume = int(mafw.get_extension_property("volume")[1])
+        
+        # Take 10 seconds to turn down the volume
+        decSteps = 10
+        volDecStep = int(volume/decSteps)
+        for i in range(1, decSteps):
+          os.system("dbus-send --session --print-reply " \
+          "--dest=com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer " \
+          "/com/nokia/mafw/renderer/gstrenderer " \
+          "com.nokia.mafw.extension.set_extension_property " \
+          "string:'volume' variant:uint32:%d &" \
+          % (volume-volDecStep*i) )
+          time.sleep(2)  
+        
         # Send pause signal to all known players
         self.pause_media_player()
         self.pause_panucci()
+        self.pause_fmradio()
 
+        # Restore the original volume 
+        os.system("dbus-send --session --print-reply " \
+        "--dest=com.nokia.mafw.renderer.Mafw-Gst-Renderer-Plugin.gstrenderer " \
+        "/com/nokia/mafw/renderer/gstrenderer " \
+        "com.nokia.mafw.extension.set_extension_property " \
+        "string:'volume' variant:uint32:%d &" \
+        % volume )
+       
         self.log('Good night...', show_banner=True)
+        
+        #Shutdown if desired
+        if self.SHUTDOWN:
+          time.sleep(1)  
+          dsmeRequest = self.systembus.get_object("com.nokia.dsme", "/com/nokia/dsme/request")
+          dsmeRequest.req_shutdown(dbus_interface="com.nokia.dsme.request")
 
         return False
 
@@ -169,7 +212,13 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
             panucci = self.bus.get_object(Panucci.DBUS_NAME, Panucci.DBUS_PATH)
             panucci.pause(dbus_interface=Panucci.DBUS_INTF)
             self.log('Panucci paused successfully.')
-    
+
+    def pause_fmradio(self):
+        if self.bus.name_has_owner(FMRadio.DBUS_NAME):
+            fmradio = self.bus.get_object(FMRadio.DBUS_NAME, FMRadio.DBUS_PATH)
+            fmradio.stop(dbus_interface=FMRadio.DBUS_INTF)
+            self.log('FM Radio paused successfully.')
+                 
     #Setup the main options/settings dialogue (the one accessible through the tool icon when in Hildon Desktop Edit mode)
     def show_options(self, widget):
         dialog = gtk.Dialog("%s Options" % APP_TITLE, None, gtk.DIALOG_DESTROY_WITH_PARENT)
@@ -233,17 +282,23 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
 
         hboxCheckboxes = gtk.HBox()
         self.iconButton = hildon.CheckButton(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.iconButton.set_label("Show icon")
+        self.iconButton.set_label("Icon")
         if self.ICON:
           self.iconButton.set_active(True)
         hboxCheckboxes.pack_start(self.iconButton)
 
         self.bannerButton = hildon.CheckButton(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
-        self.bannerButton.set_label("Show banners")
+        self.bannerButton.set_label("Banners")
         if self.SHOWBANNER:
           self.bannerButton.set_active(True)
         hboxCheckboxes.pack_start(self.bannerButton)
-                
+
+        self.shutdownButton = hildon.CheckButton(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.shutdownButton.set_label("Shutdown")
+        if self.SHUTDOWN:
+          self.shutdownButton.set_active(True)
+        hboxCheckboxes.pack_start(self.shutdownButton)
+                 
         h_button = hildon.GtkRadioButton(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, None)
         h_button.set_label("Horizontal")
         h_button.set_mode(False)
@@ -299,6 +354,11 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
            else:
              self.SHOWBANNER = False
 
+           if (self.shutdownButton.get_active()):
+             self.SHUTDOWN = True
+           else:
+             self.SHUTDOWN = False
+             
            try:
              self.PRESETS = presetsInput.get_text().split(",")
              self.PRESETS = [int(column) for column in self.PRESETS]           
@@ -314,6 +374,7 @@ class SleeperHomePlugin(hildondesktop.HomePluginItem):
              conf_client.set_string(GCONFPATH+"/layout", self.LAYOUT)
              conf_client.set_bool(GCONFPATH+"/drawicon", self.ICON)
              conf_client.set_bool(GCONFPATH+"/showbanner", self.SHOWBANNER)
+             conf_client.set_bool(GCONFPATH+"/shutdown", self.SHUTDOWN)
            except Exception:
              self.log('Unable to save settings in GConf.')
            self.draw()
